<?
// ----------------------------------------------------------------------------
// NumberCmp()
//
// Used by the SortIndex() function when we want to sort based on the message
// number (ie: it's position in the mailbox, which means we're sorting on the
// arrival time of the message)
// ----------------------------------------------------------------------------

   function NumberCmp( $a, $b )
   {
      if( $a[number] < $b[number] )
         return -1;
      else if( $a[number] > $b[number] )
         return 1;
      return 0;
   }


// ----------------------------------------------------------------------------
// DateCmp()
//
// Used by the SortIndex() function when we want to sort based on the date
// header.
// ----------------------------------------------------------------------------

   function DateCmp( $a, $b )
   {
      if( $a[date] < $b[date] )
         return -1;
      else if( $a[date] > $b[date] )
         return 1;
      return 0;
   }


// ----------------------------------------------------------------------------
// SizeCmp()
//
// Used by the SortIndex() function when we want to sort based on the message
// size.
// ----------------------------------------------------------------------------

   function SizeCmp( $a, $b )
   {
      if( $a[size] < $b[size] )
         return -1;
      else if( $a[size] > $b[size] )
         return 1;
      return 0;
   }


// ----------------------------------------------------------------------------
// FromCmp()
//
// Used by the SortIndex() function when we want to sort based on the from 
// header.
// ----------------------------------------------------------------------------

   function FromCmp( $a, $b )
   {
      return strnatcasecmp( $a[from], $b[from] );
   }


// ----------------------------------------------------------------------------
// SubjectCmp()
//
// Used by the SortIndex() function when we want to sort based on the subject
// header.  Currently it compares the subject fields as strings.  A variation
// on this might take into account any leading "Re: " tags and ignore them.
// ----------------------------------------------------------------------------

   function SubjectCmp( $a, $b )
   {
      return strnatcasecmp( $a[subject], $b[subject] );
   }


// ----------------------------------------------------------------------------
// SortIndex()
//
// This function is called by GetIndex() to sort the index array according to
// the user's sorting preference.
// ----------------------------------------------------------------------------

   function SortIndex()
   {
      global $prefs, $index, $msgcount;

      // No point sorting an array with 0 or 1 entries in it.

      if( $msgcount < 1 )
         return;

      // Sort the array using usort(), using a different comparison function
      // for each sorting option.  If the sorting option is a (descending)
      // one we also reverse the array afterwards.  Note that for the "arrival
      // time" sort option we don't have to do anything at all so we don't
      // test for it here.

      if( $prefs["sort"] == "nd" )
      {
         usort( $index, NumberCmp );
         $index = array_reverse( $index );
      }
      else if( $prefs["sort"] == "d" )
         usort( $index, DateCmp );
      else if( $prefs["sort"] == "dd" )
      {
         usort( $index, DateCmp );
         $index = array_reverse( $index );
      }
      else if( $prefs["sort"] == "f" )
         usort( $index, FromCmp );
      else if( $prefs["sort"] == "fd" )
      {
         usort( $index, FromCmp );
         $index = array_reverse( $index );
      }
      else if( $prefs["sort"] == "s" )
         usort( $index, SubjectCmp );
      else if( $prefs["sort"] == "sd" )
      {
         usort( $index, SubjectCmp );
         $index = array_reverse( $index );
      }
      else if( $prefs["sort"] == "b" )
         usort( $index, SizeCmp );
      else if( $prefs["sort"] == "bd" )
      {
         usort( $index, SizeCmp );
         $index = array_reverse( $index );
      }
   }


// ----------------------------------------------------------------------------
// GetIndex()
//
// This function retrieves a list of the available messages.  It attempts to
// cache this list so that subsequent retrievals are faster, however if the
// number or size of the available messages changes then it's forced to ask
// the POP server again.
// ----------------------------------------------------------------------------

   function GetIndex()
   {
      global $prefs, $socket, $index, $sid, $msgcount, $msgsize;

      // Clean up the index array - this is important, otherwise the index
      // will be out of whack after deleting mesages.

      while( sizeof( $index ) )
         array_pop( $index );

      // Get the current mailbox status, which tells us how many messages there
      // are and how many bytes they take up.

      fputs( $socket, "STAT\r\n" );
      $stat = chop( fgets( $socket, 500 ) );

      // Extract the number of messages and overall size of the messages from
      // the status string.

      $msgcount = intval( substr( $stat, strpos( $stat, " " ) + 1 ) );
      $msgsize = intval( substr( $stat, strrpos( $stat, " " ) + 1 ) );

      // Now we check to see if we've already cached the message index.  If
      // the index file exists and we can open it, try to use it.

      if( file_exists( "phpost_cache/$sid.idx" ) && $fp = fopen( "phpost_cache/$sid.idx", "r" ) )
      {
         // The first line of the cache file is the status string from the
         // last retrieval.  If it matches the current status string then
         // we assume that the mailbox has not changed and it is safe to
         // use the cached data.

         $line = chop( fgets( $fp, 500 ) );
         if( $stat == $line )
         {
            // Now we read in $msgcount index entries.

            for( $i = 0; $i < $msgcount; $i++ )
            {
               // The message number is easy, it's just sequential.
               $index[$i][number] = $i + 1;

               // Read in the FROM value
               $index[$i][from] = chop( fgets( $fp, 1000 ) );

               // Read in the DATE value (it's a UNIX timestamp ie: a number)
               $index[$i][date] = intval( chop( fgets( $fp, 1000 ) ) );

               // Read in the SUBJECT
               $index[$i][subject] = chop( fgets( $fp, 1000 ) );

               // Read in the SIZE (in bytes)
               $index[$i][size] = intval( chop( fgets( $fp, 1000 ) ) );

               // Read in the STATUS
               $index[$i][status] = chop( fgets( $fp, 1000 ) );

               // Read in the FLAGS
               $index[$i][flags] = chop( fgets( $fp, 1000 ) );
            }
            fclose( $fp );

            // By touching the cache file we update the timestamp on it.  This
            // keeps it "fresh" so that it doesn't get cleaned out for being
            // stale.

            touch( "phpost_cache/$sid.idx" );

            // Sort the index list and return FALSE, meaning we didn't have
            // to fetch the data from the POP server.

            SortIndex();
            return false;
         }
         // We only get here if the new status string does not match the
         // status string stored in the cache file.  In other words, we have
         // a cache file but the mailbox contents have changed.  So we
         // delete the cache file and continue on as if it wasn't there at
         // all.

         fclose( $fp );
         unlink( "phpost_cache/$sid.idx" );
      }

      // We couldn't get any data from the cache (either the cache file didn't
      // exist or it was out of date) so now we fetch each message from the
      // POP server itself.

      // First we do a LIST command and get the status and message size for
      // all the messages.

      fputs( $socket, "LIST\r\n" );
      $line = chop( fgets( $socket, 500 ) );
      if( substr( $line, 0, 1 ) == "+" )
      {
         $i = 0;
         while( ($line = chop( fgets( $socket, 500 ) )) != "." )
            $index[$i++][size] = intval( substr( $line, strrpos( $line, " " ) ) );
      }

      for( $i = 0; $i < $msgcount; $i++ )
      {
         // The message number is sequential, starting at 1.

         $n = $i + 1;
         $index[$i][number] = $n;

         // Request the top of the message (the headers) plus 0 lines of body

         fputs( $socket, "TOP $n 0\r\n" );
         $line = chop( fgets( $socket, 500 ) );
         if( substr( $line, 0, 1 ) == "+" )
         {
            // Now we'll get back lines of text containing the headers.  We
            // read until we get a line containing nothing but a period.

            $headerlist = "";
            while( ($line = chop( fgets( $socket, 1000 ) )) != "." )
            {
               $headerlist .= $line;

               // Is this line the FROM: header?

               if( strcasecmp( substr( $line, 0, 5 ), "FROM:" ) == 0 )
               {
                  // Extract the rest of the line (minus the FROM: part)
                  $f = B64QPDecode( substr( $line, 5 ) );

                  // Try to extract the text part (as opposed to the actual
                  // email address), if there is one.
                  if( ereg( "(.+) <.+@.+\..+>", $f, $r ) || ereg( ".+@.+\..+ \((.+)\)", $f, $r ) )
                  {
                     ereg( "\"(.+)\"", $r[1], $r );
                     if( trim( $r[1] ) != "" )
                        $f = $r[1];
                  }

                  if( strlen( $f ) > 20 )
                     $f = substr( $f, 0, 20 ) . "...";
                  $f = str_replace( "<", "&lt;", $f );
                  $index[$i][from] = trim( $f );
               }

               // Is this line the SUBJECT header?

               else if( strcasecmp( substr( $line, 0, 8 ), "SUBJECT:" ) == 0  )
               {
                  $s = B64QPDecode( substr( $line, 8 ) );
                  $s = str_replace( "<", "&lt;", $s );
                  $index[$i][subject] = trim( $s );
               }

               // Is this line the DATE header?

               else if( strcasecmp( substr( $line, 0, 5 ), "DATE:" ) == 0 )
               {
                  // Extract the date

                  $d = substr( $line, 5 );

                  // Eliminate "GMT" if it's present.
                  $d = ereg_replace( "\"GMT\"", "", $d );

                  // If we find any number in the date like 100, 101, 102, etc.
                  // It's likely supposed to be the year but it was mangled by
                  // a non Y2K compliant email program, so we fix it.

                  if( ereg( " (1[0-9]{2}) ", $d, $r ) )
                     $d = ereg_replace( $r[1], strval( 1900 + intval( $r[1] ) ), $d );

                  // Parse the resulting string into a UNIX time format and
                  // store it in the index array.

                  $index[$i][date] = strtotime( $d );
               }

               // Is this line the STATUS header?

               else if( strcasecmp( substr( $line, 0, 7 ), "STATUS:" ) == 0 )
                  $index[$i][status] = trim( substr( $line, 7 ) );

               // We ignore any remaining headers because we don't use them
               // in the index screen.
            }
            if( eregi( "boundary *= *[\"]*([^\r\n \"]+)", $headerlist ) )
               $index[$i][flags] = "A";
            else
               $index[$i][flags] = "";
         }
      }

      // At this point we have finished retrieving the headers from the POP
      // server.  Now we want to save the index array to a cache file to
      // speed up subsequent index retrievals.

      // First we open the cache file for writing.  Note that if we can't
      // create this file everything will still work ok, it'll just be
      // (a lot!) slower.

      if( $fp = fopen( "phpost_cache/$sid.idx", "w" ) )
      {
         // Write the overall mailbox status string first, followed by the
         // unread message count.

         fputs( $fp, "$stat\n" );

         // Then write out each entry in the index array.

         for( $i = 0; $i < $msgcount; $i++ )
         {
            $f = $index[$i][from];
            $d = $index[$i][date];
            $s = $index[$i][subject];
            $b = $index[$i][size];
            $t = $index[$i][status];
            $x = $index[$i][flags];
            fputs( $fp, "$f\n$d\n$s\n$b\n$t\n$x\n" );
         }
         fclose( $fp );

         // Change permissions to "rw-r----" (UNIX systems)

         chmod( "phpost_cache/$sid.idx", 0640 );
      }

      // Sort the index array and return TRUE, indicating that we did have
      // to fetch the data from the POP server.

      SortIndex();
      return true;
   }
?>
