<?
// ----------------------------------------------------------------------------
// SendMessage()
//
// This function sends a message.  The method it uses depends on the setting
// of $config["sendvia"].  Currently it can send via direct SMTP, by piping
// the message to sendmail, or by using an extended POP command called XMIT.
// ----------------------------------------------------------------------------

   function SendMessage()
   {
      global $socket, $vocab, $prefs, $config, $errstr, $to, $subject, $cc, $bcc, $body, $HTTP_HOST, $HTTP_POST_FILES, $encoding, $ashtml, $ccself, $references, $addressbook;

      // Make sure all the file attachments were uploaded ok...

      for( $i = 1; $i <= $prefs["attach"]; $i++ )
      {
         $HTTP_POST_FILES["attach$i"]["type"] = trim( $HTTP_POST_FILES["attach$i"]["type"] );
         $HTTP_POST_FILES["attach$i"]["name"] = trim( $HTTP_POST_FILES["attach$i"]["name"] );
         $HTTP_POST_FILES["attach$i"]["tmp_name"] = trim( $HTTP_POST_FILES["attach$i"]["tmp_name"] );

         if( $HTTP_POST_FILES["attach$i"]["name"] != "" && !is_uploaded_file( $HTTP_POST_FILES["attach$i"]['tmp_name'] ) )
         {
            $errstr = "Did not receive attachment '" . $HTTP_POST_FILES["attach$i"]["name"] . "' - most likely it was bigger than the file size limit on this server.";
            return false;
         }
      }

      LoadAddressBook();

      $to = AddressLookup( trim( $to ) );
      $cc = AddressLookup( trim( $cc ) );
      $bcc = AddressLookup( trim( $bcc ) );
      $subject = trim( $subject );
      $body = trim( $body );
      $encoding = trim( $encoding );
      $ashtml = trim( $ashtml );
      $ccself = trim( $ccself );
      $references = trim( $references );

      if( $ashtml )
         $body = str_replace( "\n", "<br>\n", $body );

      // Add the user too if they checked "BCC Self?"

      if( strcasecmp( $ccself, "on" ) == 0 )
         if( $bcc == "" )
            $bcc = $prefs["email"];
         else
            $bcc .= "," . $prefs["email"];

      // Send via PIPE (ie: pipe the message to sendmail)
      if( $config["sendvia"] == "PIPE" )
      {
         // Open a pipe to the sendmail program.  If the user's email address
         // is valid then it's safe to pass on the command line, otherwise we
         // don't pass it and live with a generic return-path

         if( ereg( "^[[:alnum:]\._-]+@[[:alnum:]\._-]+\.[[:alpha:]]+\$", $prefs["email"] ) )
            $smtp = popen( $config["sendmail"] . " -t -i -f " . $prefs["email"], "w" );
         else
            $smtp = popen( $config["sendmail"] . " -t -i", "w" );
           
         if( !$smtp )
         {
            $errstr = "Could not launch sendmail";
            return false;
         }
      }

      // Send via the extended POP XMIT command (qpopper only I believe)
      else if( $config["sendvia"] == "XMIT" )
      {
         $smtp = $socket;
         fputs( $smtp, "XTND XMIT\r\n" );
     
         $line = fgets( $socket, 500 );
         if( substr( $line, 0, 1 ) != "+" )
         {
            $errstr = substr( $line, 5 );
            return false;
         }
      }

      // Send via direct SMTP
      else
      {
         $smtp = fsockopen( $config["smtp"], 25, $errno, $errstr, 15 );
         if( !$smtp )
         {
            $errstr = "Connection refused";
            return false;
         }

         // Get the SMTP welcome banner.  If it doesn't start with "220" then
         // it isn't available to accept any messages.

         $errstr = fgets( $smtp, 500 );
         if( substr( $errstr, 0, 3 ) != 220 )
            return false;

         // Send HELO command and check for a 250 result code.

         fputs( $smtp, "HELO $HTTP_HOST\r\n" );
         $errstr = fgets( $smtp, 500 );
         if( substr( $errstr, 0, 3 ) != 250 )
            return false;

         // Issue the MAIL FROM command and check for a 250 result code.

         fputs( $smtp, "MAIL FROM: <" . $prefs["email"] . ">\r\n" );
         $errstr = fgets( $smtp, 500 );
         if( substr( $errstr, 0, 3 ) != 250 )
            return false;

         // Build a list of recipients from the TO, CC, and BCC values.

         $recipients = $to . "," . $cc . "," . $bcc;

         // Split the recipient list up into an array (using spaces and commas
         // as the delimiters

         $x = split( "[,]+", $recipients );

         // Loop through each recipient...

         $n = 0;
         for( $i = 0; $i < sizeof($x); $i++ )
         {
            $t = $x[$i];

            // Delete comments in parenthesis...
            $t = ereg_replace( "\(.*\)", "", $t );

            // Look for email address enclosed in '<' and '>'...
            if( ereg( "\<(.+)\>", $t, $reg ) )
               $t = $reg[1];

            // If the recipient is not a null value, issue a RCPT TO command
            // and check that the result code is 250 and increment a counter.

            if( $t != "" )
            {
               fputs( $smtp, "RCPT TO: <$t>\r\n" );
               $errstr = fgets( $smtp, 500 );
               if( substr( $errstr, 0, 2 ) != 25 )
                  return false;
               $n++;
            }
         }

         // Issue the DATA command and expect a 354 result code.
   
         fputs( $smtp, "DATA\r\n" );
         $errstr = fgets( $smtp, 500 );
         if( substr( $errstr, 0, 3 ) != 354 )
            return false;
      }

      // Now we send the message headers.

      fputs( $smtp, "Reply-To: " . $prefs["email"] . "\r\n" );
      fputs( $smtp, "From: " . $prefs["email"] );
      if( $prefs["name"] != "" )
         fputs( $smtp, " (" . $prefs["name"] . ")" );
      fputs( $smtp, "\r\n" );
      fputs( $smtp, "To: $to\r\n" );

      // If the subject is empty we add a default one.

      if( strlen( $subject ) > 0 )
         fputs( $smtp, "Subject: $subject\r\n" );
      else
         fputs( $smtp, "Subject: No subject\r\n" );

      // Only add a CC header if a CC value was entered.

      if( strlen( $cc ) > 2 )
         fputs( $smtp, "Cc: $cc\r\n" );

      if( $config["sendvia"] != "SMTP" && strlen( $bcc ) > 2 )
         fputs( $smtp, "Bcc: $bcc\r\n" );

      // Generate the Date: header.

      $tzo = intval( date( "Z" ) ) / 36;
      $d = date( "D, j M Y H:i:s " ) . ($tzo < 0 ? "" : "+") . sprintf( "%04d", $tzo );
      fputs( $smtp, "Date: $d\r\n" );

      // Now the message-id header.

      fputs( $smtp, "Message-Id: <" . uniqid( "", true ) . "@" . $config["domain"] . ">\r\n" );

      // Add a references header if this is a reply.

      if( $references != "" )
         fputs( $smtp, "References: $references\r\n" );

      // You may want to comment out these next three lines.  They insert
      // your IP address into the headers.  This is useful/needed if you
      // want to be able to track down abusers of your system, but you
      // may want to be more anonymous, it's up to you.

      fputs( $smtp, "X-Authenticated-IP: [" . getenv( "REMOTE_ADDR" ) . "]\r\n" );

      // Insert the sender's username and POP server for the same reasons
      // as above:

      fputs( $smtp, "X-Sender: " . $prefs["username"] . "@" . $prefs["pop"] . "\r\n" );


      fputs( $smtp, "X-Mailer: PHPost " . $config["version"] . " (http://webgadgets.com/phpost/)\r\n" );

      if( isset( $config["extraheaders"] ) )
         fputs( $smtp, $config["extraheaders"] . "\r\n" );

      // Deal with attachments (if there are any)

      for( $i = 1; $i <= $prefs["attach"]; $i++ )
      {
         if( $HTTP_POST_FILES["attach$i"]['name'] != "" && is_uploaded_file( $HTTP_POST_FILES["attach$i"]['tmp_name'] ) )
         {
            $attachment = true;

            $boundary = "=_" . uniqid( "", true ) . "=_";

            break;
         }
      }

      fputs( $smtp, "MIME-version: 1.0\r\n" );
      if( $attachment )
         fputs( $smtp, "Content-type: multipart/mixed; boundary=\"$boundary\";\r\n" );
      else if( $ashtml )
         fputs( $smtp, "Content-type: text/html; charset=\"" . $vocab["charset"] . "\"\r\n" );
      else
         fputs( $smtp, "Content-type: text/plain; charset=\"" . $vocab["charset"] . "\"\r\n" );

      // We signal the end of the headers with an empty line.

      fputs( $smtp, "\r\n" );

      if( $attachment )
      {
         fputs( $smtp, str_replace( "\n", "\r\n", $vocab["mimecomment"] ) );
         fputs( $smtp, "\r\n\r\n" );
      }

      if( $body != "" )
      {
         if( $attachment  )
         {
            fputs( $smtp, "--$boundary\r\n" );
            fputs( $smtp, "Content-Type: " );
            if( $ashtml )
               fputs( $smtp, "text/html" );
            else
               fputs( $smtp, "text/plain" );
            fputs( $smtp, "; charset=\"" . $vocab["charset"] . "\"\r\n\r\n" );
         }

         if( $config["sendvia"] != "PIPE" )
            $body = str_replace( "\n.", "\n..", $body );
         fputs( $smtp, stripslashes( $body ) );
         //fputs( $smtp, stripslashes( str_replace( "\n", "\r\n", $body ) ) );
         fputs( $smtp, "\r\n" );

         if( $attachment  )
            fputs( $smtp, "\r\n\r\n" );

      }
      if( $attachment )
      {
         for( $i = 1; $i <= $prefs["attach"]; $i++ )
         {
            if( $HTTP_POST_FILES["attach$i"]['name'] != "" && is_uploaded_file( $HTTP_POST_FILES["attach$i"]['tmp_name'] ) )
            {
               if( strcasecmp( $HTTP_POST_FILES["attach$i"]['type'], "text/plain" ) && strcasecmp( $HTTP_POST_FILES["attach$i"]['type'], "text/html" ) )
                  $encode = true;
               else
                  $encode = false;

               fputs( $smtp, "--$boundary\r\nContent-Type: " );
               fputs( $smtp, $HTTP_POST_FILES["attach$i"]['type'] );
               fputs( $smtp, "\r\n" );
               if( $encode )
               {
                  if( $encoding == "uuencoding" )
                     fputs( $smtp, "Content-Transfer-Encoding: x-uuencode\r\n" );
                  else
                     fputs( $smtp, "Content-Transfer-Encoding: base64\r\n" );
               }
               fputs( $smtp, "Content-Disposition: attachment; filename=\"" );
               fputs( $smtp, $HTTP_POST_FILES["attach$i"]['name'] );
               fputs( $smtp, "\"\r\n\r\n" );

               $newtmp = "phpost_cache/" . uniqid( "", true ) . ".tmp";
               move_uploaded_file( $HTTP_POST_FILES["attach$i"]['tmp_name'], $newtmp );
               $HTTP_POST_FILES["attach$i"]['tmp_name'] = $newtmp;
               chmod( $newtmp, 0644 );

               if( $fp = fopen( $HTTP_POST_FILES["attach$i"]['tmp_name'], "r" ) )
               {
                  if( $encode )
                  {
                     if( $encoding == "uuencoding" )
                     {
                        fputs( $smtp, "begin 600 " );
                        fputs( $smtp, $HTTP_POST_FILES["attach$i"]['name'] );
                        fputs( $smtp, "\r\n" );
                        while( $line = fread( $fp, 45 ) )
                        {
                           fputs( $smtp, UUEncode( $line, "uu" ) );
                           fputs( $smtp, "\r\n" );
                        }
                        fputs( $smtp, "`\r\nend\r\n" );
                     }
                     else while( $line = fread( $fp, 54 ) )
                     {
                        fputs( $smtp, base64_encode( $line ) );
                        fputs( $smtp, "\r\n" );
                     }
                  }
                  else while( $line = fgets( $fp, 1024 ) )
                     fputs( $smtp, $line );
                  fputs( $smtp, "\r\n" );
                  fclose( $fp );
                  unlink( $HTTP_POST_FILES["attach$i"]['tmp_name'] );
               }
            }
         }
         fputs( $smtp, "--$boundary--\r\n" );
      }

      // We signal the end of the body by sending a period on a line by itself.
      // (but not for the PIPE style of sending...)

      if( $config["sendvia"] != "PIPE" )
         fputs( $smtp, "\r\n.\r\n" );

      // Check the result code.  if it's not a 250 then it wasn't sent.

      $errstr = fgets( $smtp, 500 );

      if( $config["sendvia"] == "SMTP" )
      {
         fclose( $smtp );
         if( substr( $errstr, 0, 3 ) != 250 )
            return false;
      }
      else if( $config["sendvia"] == "PIPE" )
         pclose( $smtp );

      // Success!

      return true;
   }
?>
